/** 
* @file llfloatertestlistview.cpp
* @brief Tests of programmatic manipulation of LLListView widgets
*
* $LicenseInfo:firstyear=2009&license=viewergpl$
* 
* Copyright (c) 2009, Linden Research, Inc.
* 
* Second Life Viewer Source Code
* The source code in this file ("Source Code") is provided by Linden Lab
* to you under the terms of the GNU General Public License, version 2.0
* ("GPL"), unless you have obtained a separate licensing agreement
* ("Other License"), formally executed by you and Linden Lab.  Terms of
* the GPL can be found in doc/GPL-license.txt in this distribution, or
* online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
* 
* There are special exceptions to the terms and conditions of the GPL as
* it is applied to this Source Code. View the full text of the exception
* in the file doc/FLOSS-exception.txt in this software distribution, or
* online at
* http://secondlifegrid.net/programs/open_source/licensing/flossexception
* 
* By copying, modifying or distributing this software, you acknowledge
* that you have read and understood your obligations described above,
* and agree to abide by those obligations.
* 
* ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
* WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
* COMPLETENESS OR PERFORMANCE.
* $/LicenseInfo$
*/
#include "llviewerprecompiledheaders.h"

#include "llfloatertestlistview.h"

// Viewer includes
#include "lllistview.h"

// Linden library includes
//#include "lluictrlfactory.h"

LLFloaterTestListView::LLFloaterTestListView(const LLSD& seed)
:	LLFloater(seed),
	mListView(NULL)
{
	// set up named callback functions for test buttons
	mCommitCallbackRegistrar.add("TestListView.Test1",
		boost::bind(&LLFloaterTestListView::onClickTest1, this));
	mCommitCallbackRegistrar.add("TestListView.Test2",
		boost::bind(&LLFloaterTestListView::onClickTest2, this));
}

LLFloaterTestListView::~LLFloaterTestListView()
{}

BOOL LLFloaterTestListView::postBuild()
{
	mListView = getChild<LLListView>("test_list_view");
	// just set a random property
	mListView->setString("set programmatically");
	return LLFloater::postBuild();
}

void LLFloaterTestListView::onListViewChanged()
{
	llinfos << "list view changed" << llendl;
}

void LLFloaterTestListView::onClickTest1()
{
	llinfos << "test 1" << llendl;
}

void LLFloaterTestListView::onClickTest2()
{
	llinfos << "test 2" << llendl;
}
